

/**
  ******************************************************************************
  * @file    TIM/PWM_Output/stm32f10x_it.c 
  * @author  MCD Application Team
  * @version V3.5.0
  * @date    08-April-2011
  * @brief   Main Interrupt Service Routines.
  *         This file provides template for all exceptions handler and
  *         peripherals interrupt service routine.
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */ 

/* Includes ------------------------------------------------------------------*/
#include "stm32f10x_it.h"

#define	treshold	30

/** @addtogroup STM32F10x_StdPeriph_Examples
  * @{
  */

/** @addtogroup TIM_PWM_Output
  * @{
  */ 

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
uint16_t pwmleddutycycle=0; //voor LED dimmer
uint8_t pulsinit=0; //voor initieeren 1 afstandsmeting
uint16_t dir=100; //voor LED dimmer: dimmer omhoog of omlaag?
extern __IO uint16_t CCR1_Val;
extern __IO uint16_t CCR2_Val;
extern __IO uint16_t CCR3_Val;
extern __IO uint16_t CCR4_Val;

uint16_t verschilwaarde=0;
uint16_t eindwaarde = 0;
volatile uint16_t fase = 0;
volatile uint16_t afstand = 0; //variabele is hier niet lokaal dus geen extern, __IO is defined als volatile is ie hier ook niet
extern __IO uint8_t continuemeten; 
uint16_t vorigeafstand=0;
volatile extern uint8_t metingloopt;
volatile uint8_t obstakel=0;

//debugvariabelen, tijdelijk
uint16_t aantalinterrupts,irqpinstatus=0;
volatile uint16_t goedafgehandeld,foutafgehandeld=0;

//hopelijk werkt dit zo met EXTERN want deze struct staat in main.c al gedefined
extern TIM_ICInitTypeDef  TIM15_ICInitStructure;  //voor TIM15 instellen voor terugmeten SR-CH4 echopulse 
extern void meetafstand(void);
extern void debugled(uint8_t aanuit);
extern void HCSR04trig(void);
extern void setTIM2CHxDuty(uint16_t channel, uint16_t duty);
extern void ontwijk(void);

/******************************************************************************/
/*            Cortex-M3 Processor Exceptions Handlers                         */
/******************************************************************************/

/**
  * @brief  This function handles NMI exception.
  * @param  None
  * @retval None
  */
void NMI_Handler(void)
{
}

/**
  * @brief  This function handles Hard Fault exception.
  * @param  None
  * @retval None
  */
void HardFault_Handler(void)
{
  /* Go to infinite loop when Hard Fault exception occurs */
  while (1)
  {}
}

/**
  * @brief  This function handles Memory Manage exception.
  * @param  None
  * @retval None
  */
void MemManage_Handler(void)
{
  /* Go to infinite loop when Memory Manage exception occurs */
  while (1)
  {}
}

/**
  * @brief  This function handles Bus Fault exception.
  * @param  None
  * @retval None
  */
void BusFault_Handler(void)
{
  /* Go to infinite loop when Bus Fault exception occurs */
  while (1)
  {}
}

/**
  * @brief  This function handles Usage Fault exception.
  * @param  None
  * @retval None
  */
void UsageFault_Handler(void)
{
  /* Go to infinite loop when Usage Fault exception occurs */
  while (1)
  {}
}

/**
  * @brief  This function handles Debug Monitor exception.
  * @param  None
  * @retval None
  */
void DebugMon_Handler(void)
{}

/**
  * @brief  This function handles SVCall exception.
  * @param  None
  * @retval None
  */
void SVC_Handler(void)
{}

/**
  * @brief  This function handles PendSV_Handler exception.
  * @param  None
  * @retval None
  */
void PendSV_Handler(void)
{}

/**
  * @brief  This function handles SysTick Handler.
  * @param  None
  * @retval None
  */
void SysTick_Handler(void)
{
  //TimingDelay_Decrement();
}
/******************************************************************************/
/*                 STM32F10x Peripherals Interrupt Handlers                   */
/*  Add here the Interrupt Handler for the used peripheral(s) (PPP), for the  */
/*  available peripheral interrupt handler's name please refer to the startup */
/*  file (startup_stm32f10x_xx.s).                                            */
/******************************************************************************/


//irq gaat om de zoveel tijd af, namelijk Fclk / CCR4_Val
//Fclk is 24Mhz voor mijn board en CCR4_Val zie define in main is 6826
//Dat betekent om de 284uS
void TIM2_IRQHandler(void)
{
  if (TIM_GetITStatus(TIM2, TIM_IT_CC4) == SET)
  {
    TIM_ClearITPendingBit(TIM2, TIM_IT_CC4); //clear de interrupt van CH4

	//voor body leds pwm-en
	pwmleddutycycle+=dir;	 //dir start incrementen dus led feller
	setTIM2CHxDuty(1,pwmleddutycycle); 	
	if(pwmleddutycycle==65500)
		dir=-dir;//nu decrementen dus dimmen
	if(pwmleddutycycle==0)
		dir=-dir;//en weer van voren af aan dus incrementen	 

	//voor metingen doen met SR-HC04 sensor
	if ((pulsinit==20) && (continuemeten==1)) //pulsinit 20 gaf op scope 1x meting per 55mS
	{
		HCSR04trig(); //trigger een afstandsmeting
		pulsinit=0;
	}
	pulsinit++;			
  }
}
  
//hier wordt op de opgaande flank van de echopuls in fase 0 de TIM waarde gecaptured
//idem in neergaande flank en fase 1, het verschil aantal tellen is het aantal uS die de puls duurde
//afstand is dan dat verschil delen door 58.
//metingloopt=0 geeft aan dat 1 meting volledig klaar is, zo kan deze IRQ de triggerfunctie besturen en zorgen
//dat er alleen een meting wordt gestart als de vorige klaar is
void TIM1_BRK_TIM15_IRQHandler(void)
{ 
	if((TIM_GetITStatus(TIM15, TIM_IT_CC1) == SET)) //Check of interrupt komt van CC1 dus flank detected
  	{
		irqpinstatus=GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_14); //check of pin14 na de flank hoog of laag is
		//aantalinterrupts++;
		TIM_ClearITPendingBit(TIM15, TIM_IT_CC1); //ja dus reset t bit meteen
		
		if((metingloopt==1) && (((fase == 0) && (irqpinstatus==0)) | ((fase == 1) && (irqpinstatus==1))))	//foutsituaties
		{	foutafgehandeld++;
			afstand=0;
			metingloopt=0;
		}

		if((fase == 0) && (irqpinstatus==1))//check of we in fase 0 zitten EN opgaande flank (PB14 is dan 1 geworden)
    	{
			TIM15->CNT=0; //timer ff resetten
			TIM_ITConfig(TIM15, TIM_IT_CC2, ENABLE);  //interrupt als TIM15 overflowt
			TIM_ClearITPendingBit(TIM15, TIM_IT_CC2); //als cc2 nu in fase 1 weer geset blijkt te zijn dan is TIM15 overflowed
      		fase = 1;	 
    	}

		if((fase == 1) && (irqpinstatus==0)) ////check of we in fase 1 zijn EN neergaande flank (PB14 is dan 0)
    	{
			TIM_ITConfig(TIM15, TIM_IT_CC2, DISABLE);
			eindwaarde = TIM_GetCapture1(TIM15); //in fase 1 was TIM gereset, wat is tim stand in fase 1?	
      			
			if((TIM_GetITStatus(TIM15, TIM_IT_CC2) == SET)| eindwaarde > 29000) //als CC2IF geset is of eindwaarde >29000 (is >500cm afstand) dan is de meting ongeldig
			{
        		afstand=0; //afstandwaarde ongeldig omdat TIM overflow is opgetreden, het gaat dus om een bogey waarde	
				TIM_ClearITPendingBit(TIM15, TIM_IT_CC2);
			}
      		else
				afstand = eindwaarde/58;  //afstand in cm is aantal getelde pulsen (waarvan ieder 1 uS) delen door decimaal 58		
			
			fase = 0;
			metingloopt=0; 
			goedafgehandeld++;
			vorigeafstand=afstand;

			if ((afstand<treshold) && (afstand>2)) 	//treshold is define, 2 is veilige limiet
				obstakel=1;
      	}
	}	   
}

